==> hermes-5.4.0.3.tar.gz: hermes-5.4.0.3/spec/apis/dmo/entity_codes_spec.rb <==
require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  MUTATE_PARAMS = {
    code: 'String',
    expire_at: 'DateTime',
    applicant_id: 'BSON::ObjectId',
    entity_id: 'BSON::ObjectId'
  }

  ACCEPTABLE_PARAMS = %w[
    code
    expire_at
    applicant_id
    entity_id
    created_at
    updated_at
  ]

  RSpec.shared_examples 'entreprise only' do
    before do
      authenticated_header(request, user)
      subject
    end
    it 'returns unauthorized error' do
      expect(response).to have_http_status 401
    end
  end

  describe Api::Dmo::EntityCodesController, :type => :api do
    routes { Hermes::Engine.routes }
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:entreprise) { create(:user, roles: ['entreprise']) }
    let!(:roles) { %w[reservataire bailleur demandeur Manager Supervisor Administrator] }
    let!(:user) { create(:user, roles: roles.sample(rand(1..3))) }
    let!(:demandeur) { create(:user, auth_roles: ['demandeur']) }
    let!(:entity_code) { create :entity_code }

    before { authenticated_header(request, entreprise) }

    describe 'GET index' do
      let!(:entity_codes) { create_list :entity_code, 4 }

      before do
        authenticated_header(request, demandeur)
        get 'index'
      end

      it 'returns status code 200' do
        expect(response).to have_http_status 200
      end
      it 'returns a valid json' do
        expect(json['data'].length).to eq 5
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq 5
      end
    end

    describe 'POST create' do
      let(:entity) { create :entity }
      let(:applicant) { create :applicant }
      let(:params) do
        { attributes: { entity_id: entity.id, applicant_id: applicant.id } }
      end
      before do
        allow(SecureRandom).to receive(:alphanumeric).with(8).and_return 'abcd1234'
      end

      subject { post 'create', { data: params } }

      it_behaves_like 'entreprise only'

      it 'succeeds' do
        subject
        expect(response).to have_http_status 201
      end

      it 'creates an entity_code' do
        expect { subject }.to change { Hermes::EntityCode.count }.by 1
      end

      context 'when no code specified' do
        it 'returns a valid json' do
          subject
          expect(json['data']['attributes']['code']).to eq 'ABCD1234'
          expect(json['data']['attributes']['applicant_id']).to eq applicant.id.to_s
          expect(json['data']['attributes']['entity_id']).to eq entity.id.to_s
        end
      end

      context 'when code is specified' do
        before do
          params[:attributes].merge!(code: 'Specified_CODE')
          subject
        end
        it 'returns a valid json' do
          expect(json['data']['attributes']['code']).to eq 'Specified_CODE'
          expect(json['data']['attributes']['applicant_id']).to eq applicant.id.to_s
          expect(json['data']['attributes']['entity_id']).to eq entity.id.to_s
        end
      end
    end

    describe 'DELETE destroy' do
      let!(:owned_entity_code) { create :entity_code, user_id: entreprise.id }

      subject { delete 'destroy', { id: owned_entity_code.id } }

      it_behaves_like 'entreprise only'

      context 'when entity_code owned' do
        it 'succeeds' do
          subject
          expect(response).to have_http_status 204
        end
        it 'destroys the entity_code' do
          expect { subject }.to change { Hermes::EntityCode.count }.by -1
        end
      end

      context 'when entity_code not owned' do
        subject { delete 'destroy', { id: entity_code.id } }
        it 'returns error' do
          subject
          expect(response).to have_http_status 401
        end
      end
    end

    describe 'POST check_entity_code_validity' do
      let!(:expired_code) { create :entity_code, code: 'ABCD1234' }
      let!(:valid_code) { create :entity_code, code: 'DEFG5678' }

      before { expired_code.set(expire_at: Time.zone.now - 1.month) }

      it 'requires code param' do
        post 'check_entity_code_validity'
        expect(response).to have_http_status 406
        expect(json['success']).to eq false
        expect(json['message']).to eq 'param is missing or the value is empty: code'
      end

      it 'returns code_not_found error' do
        post 'check_entity_code_validity', code: 'WXYZ0987'
        expect(response).to have_http_status 404
        expect(json['success']).to eq false
        expect(json['message']).to eq "Le code 'WXYZ0987' n'existe pas."
      end

      it 'returns expired_code error' do
        post 'check_entity_code_validity', code: 'ABCD1234'
        expect(response).to have_http_status 422
        expect(json['success']).to eq false
        expect(json['message']).to eq "Le code 'ABCD1234' est expiré."
      end

      it 'succeeds' do
        post 'check_entity_code_validity', code: 'DEFG5678'
        expect(response).to have_http_status 200
        expect(json['success']).to eq true
        expect(json['code']).to eq 'DEFG5678'
        expect(json['message']).to eq 'Code valide et non expiré.'
      end
    end
  end

  require_relative '../../swagger_helper'

  # describe Hermes::Api::Dmo::EntityCodesController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  #   TAGS_API_DMO_ENTITY_CODES = 'API_DMO_ENTITY_CODES'

  #   include JwtAuthHelper
  #   extend HermesSwaggerJsonParams
  #   include SwaggerJsonAttributes
  #   let!(:entreprise) { create(:user, roles: ['entreprise']) }

  #   context 'as entreprise' do
  #     path '/api/dmo/entity_codes' do
  #       post 'Create entity code' do
  #         tags TAGS_API_DMO_ENTITY_CODES
  #         produces 'application/json'
  #         consumes 'application/json'
  #         security [Bearer: {}]
  #         json_params = {}
  #         json_params[:applicant_id] = { type: :string, required: true }
  #         json_params[:entity_id] = { type: :string, required: true }
  #         json_params[:expire_at] = { type: :string, format: 'date-time' }

  #         parameter name: :params, in: :body, schema: jsonify_params(json_params)

  #         response '201', 'entity code created' do
  #           let(:Authorization) { "Bearer #{token_for(entreprise)}" }
  #           let(:params) do
  #             jsonapi_attributes(
  #                { applicant_id: 'String', entity_id: 'String' }
  #             )
  #           end
  #           include_context 'with integration test'
  #           include_context 'valid jsonapi'
  #           include_context 'not containings errors'

  #           after do |_example|
  #             json = JSON.parse(response.body)

  #             expect(json['data']).to be_a Hash
  #             expect(json['data']['id']).to_not be nil
  #             expect(json['data']['type']).to eq 'entity_codes'
  #           end
  #         end
  #       end
  #     end

  #     path '/api/dmo/entity_codes' do
  #       get 'Retrieves all entity codes' do
  #         tags TAGS_API_DMO_ENTITY_CODES
  #         produces 'application/json'

  #         ACCEPTABLE_PARAMS.each do |acceptable_param|
  #           type = ''
  #           if MUTATE_PARAMS.include?(acceptable_param)
  #             type = nice_type(MUTATE_PARAMS[acceptable_param])
  #           end
  #           parameter name: acceptable_param.to_s, in: :query, type: type, required: false
  #         end

  #         security [Bearer: {}]
  #         paginator

  #         response '200', 'entity codes found' do
  #           let(:Authorization) { "Bearer #{token_for(entreprise)}" }
  #           let!(:entity_codes) { create_list(:entity_code, 5) }

  #           include_context 'with integration test'
  #           include_context 'valid jsonapi collection' do
  #             let(:expected_collection) { entity_codes }
  #           end
  #         end
  #       end
  #     end

  #     path '/api/dmo/entity_codes/{id}' do
  #       delete 'Delete a single entity code by id' do
  #         tags TAGS_API_DMO_ENTITY_CODES
  #         produces 'application/json'
  #         parameter name: :id, in: :path, type: :string
  #         security [Bearer: {}]

  #         response '204', 'entity code deleted' do
  #           let(:Authorization) { "Bearer #{token_for(entreprise)}" }
  #           let(:id) { create(:entity_code, user_id: entreprise.id).id }
  #           run_test!
  #         end
  #       end
  #     end

  #     path '/api/dmo/entity_codes/check_entity_code_validity' do
  #       post 'Check entity_code validity by code' do
  #         tags TAGS_API_DMO_ENTITY_CODES
  #         produces 'application/json'
  #         consumes 'application/json'
  #         parameter name: :params, in: :body, schema: {
  #           type: :object,
  #           properties: {
  #             code: { type: :string },
  #             required: [:code]
  #           }
  #         }
  #         security [Bearer: {}]

  #         response '200', 'entity code checked' do
  #           let(:Authorization) { "Bearer #{token_for(entreprise)}" }
  #           let!(:entity_code) { create(:entity_code, code: 'ABCD1234') }
  #           let(:params) do
  #             { code: 'ABCD1234' }
  #           end

  #           include_context 'with integration test'
  #           include_context 'not containings errors'
  #           after do |_example|
  #             json = JSON.parse(response.body)
  #             expect(json['success']).to eq true
  #             expect(json['code']).to eq 'ABCD1234'
  #             expect(json['message']).to eq 'Code valide et non expiré.'
  #           end
  #         end
  #       end
  #     end
  #   end
  # end
end

==> hermes-5.5.0.1.tar.gz: hermes-5.5.0.1/spec/apis/dmo/entity_codes_spec.rb <==
require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  MUTATE_PARAMS = {
    code: 'String',
    expire_at: 'DateTime',
    applicant_id: 'BSON::ObjectId',
    entity_id: 'BSON::ObjectId'
  }

  ACCEPTABLE_PARAMS = %w[
    code
    expire_at
    applicant_id
    entity_id
    created_at
    updated_at
  ]

  RSpec.shared_examples 'entreprise only' do
    before do
      authenticated_header(request, user)
      subject
    end
    it 'returns unauthorized error' do
      expect(response).to have_http_status 401
    end
  end

  describe Api::Dmo::EntityCodesController, :type => :api do
    routes { Hermes::Engine.routes }
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper

    let!(:entreprise) { create(:user, roles: ['entreprise']) }
    let!(:roles) { %w[bailleur demandeur Manager Supervisor Administrator] }
    let!(:user) { create(:user, roles: roles.sample(rand(1..3))) }
    let!(:demandeur) { create(:user, auth_roles: ['demandeur']) }
    let!(:entity_code) { create :entity_code }

    before { authenticated_header(request, entreprise) }

    describe 'GET index' do
      let!(:entity_codes) { create_list :entity_code, 4 }

      before do
        authenticated_header(request, demandeur)
        get 'index'
      end

      it 'returns status code 200' do
        expect(response).to have_http_status 200
      end
      it 'returns a valid json' do
        expect(json['data'].length).to eq 5
        expect(json['meta']['pagination']).to be_present
        expect(json['meta']['pagination']['total_objects']).to eq 5
      end
    end

    describe 'POST create' do
      let(:entity) { create :entity }
      let(:applicant) { create :applicant }
      let(:params) do
        { attributes: { entity_id: entity.id, applicant_id: applicant.id } }
      end
      before do
        allow(SecureRandom).to receive(:alphanumeric).with(8).and_return 'abcd1234'
      end

      subject { post 'create', { data: params } }

      it_behaves_like 'entreprise only'

      it 'succeeds' do
        subject
        expect(response).to have_http_status 201
      end

      it 'creates an entity_code' do
        expect { subject }.to change { Hermes::EntityCode.count }.by 1
      end

      context 'when no code specified' do
        it 'returns a valid json' do
          subject
          expect(json['data']['attributes']['code']).to eq 'ABCD1234'
          expect(json['data']['attributes']['applicant_id']).to eq applicant.id.to_s
          expect(json['data']['attributes']['entity_id']).to eq entity.id.to_s
        end
      end

      context 'when code is specified' do
        before do
          params[:attributes].merge!(code: 'Specified_CODE')
          subject
        end
        it 'returns a valid json' do
          expect(json['data']['attributes']['code']).to eq 'Specified_CODE'
          expect(json['data']['attributes']['applicant_id']).to eq applicant.id.to_s
          expect(json['data']['attributes']['entity_id']).to eq entity.id.to_s
        end
      end
    end

    describe 'DELETE destroy' do
      let!(:owned_entity_code) { create :entity_code, user_id: entreprise.id }

      subject { delete 'destroy', { id: owned_entity_code.id } }

      it_behaves_like 'entreprise only'

      context 'when entity_code owned' do
        it 'succeeds' do
          subject
          expect(response).to have_http_status 204
        end
        it 'destroys the entity_code' do
          expect { subject }.to change { Hermes::EntityCode.count }.by -1
        end
      end

      context 'when entity_code not owned' do
        subject { delete 'destroy', { id: entity_code.id } }
        it 'returns error' do
          subject
          expect(response).to have_http_status 401
        end
      end
    end

    describe 'POST check_entity_code_validity' do
      let!(:expired_code) { create :entity_code, code: 'ABCD1234' }
      let!(:valid_code) { create :entity_code, code: 'DEFG5678' }

      before { expired_code.set(expire_at: Time.zone.now - 1.month) }

      it 'requires code param' do
        post 'check_entity_code_validity'
        expect(response).to have_http_status 406
        expect(json['success']).to eq false
        expect(json['message']).to eq 'param is missing or the value is empty: code'
      end

      it 'returns code_not_found error' do
        post 'check_entity_code_validity', code: 'WXYZ0987'
        expect(response).to have_http_status 404
        expect(json['success']).to eq false
        expect(json['message']).to eq "Le code 'WXYZ0987' n'existe pas."
      end

      it 'returns expired_code error' do
        post 'check_entity_code_validity', code: 'ABCD1234'
        expect(response).to have_http_status 422
        expect(json['success']).to eq false
        expect(json['message']).to eq "Le code 'ABCD1234' est expiré."
      end

      it 'succeeds' do
        post 'check_entity_code_validity', code: 'DEFG5678'
        expect(response).to have_http_status 200
        expect(json['success']).to eq true
        expect(json['code']).to eq 'DEFG5678'
        expect(json['message']).to eq 'Code valide et non expiré.'
      end
    end
  end

  require_relative '../../swagger_helper'

  # describe Hermes::Api::Dmo::EntityCodesController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  #   TAGS_API_DMO_ENTITY_CODES = 'API_DMO_ENTITY_CODES'

  #   include JwtAuthHelper
  #   extend HermesSwaggerJsonParams
  #   include SwaggerJsonAttributes
  #   let!(:entreprise) { create(:user, roles: ['entreprise']) }

  #   context 'as entreprise' do
  #     path '/api/dmo/entity_codes' do
  #       post 'Create entity code' do
  #         tags TAGS_API_DMO_ENTITY_CODES
  #         produces 'application/json'
  #         consumes 'application/json'
  #         security [Bearer: {}]
  #         json_params = {}
  #         json_params[:applicant_id] = { type: :string, required: true }
  #         json_params[:entity_id] = { type: :string, required: true }
  #         json_params[:expire_at] = { type: :string, format: 'date-time' }

  #         parameter name: :params, in: :body, schema: jsonify_params(json_params)

  #         response '201', 'entity code created' do
  #           let(:Authorization) { "Bearer #{token_for(entreprise)}" }
  #           let(:params) do
  #             jsonapi_attributes(
  #                { applicant_id: 'String', entity_id: 'String' }
  #             )
  #           end
  #           include_context 'with integration test'
  #           include_context 'valid jsonapi'
  #           include_context 'not containings errors'

  #           after do |_example|
  #             json = JSON.parse(response.body)

  #             expect(json['data']).to be_a Hash
  #             expect(json['data']['id']).to_not be nil
  #             expect(json['data']['type']).to eq 'entity_codes'
  #           end
  #         end
  #       end
  #     end

  #     path '/api/dmo/entity_codes' do
  #       get 'Retrieves all entity codes' do
  #         tags TAGS_API_DMO_ENTITY_CODES
  #         produces 'application/json'

  #         ACCEPTABLE_PARAMS.each do |acceptable_param|
  #           type = ''
  #           if MUTATE_PARAMS.include?(acceptable_param)
  #             type = nice_type(MUTATE_PARAMS[acceptable_param])
  #           end
  #           parameter name: acceptable_param.to_s, in: :query, type: type, required: false
  #         end

  #         security [Bearer: {}]
  #         paginator

  #         response '200', 'entity codes found' do
  #           let(:Authorization) { "Bearer #{token_for(entreprise)}" }
  #           let!(:entity_codes) { create_list(:entity_code, 5) }

  #           include_context 'with integration test'
  #           include_context 'valid jsonapi collection' do
  #             let(:expected_collection) { entity_codes }
  #           end
  #         end
  #       end
  #     end

  #     path '/api/dmo/entity_codes/{id}' do
  #       delete 'Delete a single entity code by id' do
  #         tags TAGS_API_DMO_ENTITY_CODES
  #         produces 'application/json'
  #         parameter name: :id, in: :path, type: :string
  #         security [Bearer: {}]

  #         response '204', 'entity code deleted' do
  #           let(:Authorization) { "Bearer #{token_for(entreprise)}" }
  #           let(:id) { create(:entity_code, user_id: entreprise.id).id }
  #           run_test!
  #         end
  #       end
  #     end

  #     path '/api/dmo/entity_codes/check_entity_code_validity' do
  #       post 'Check entity_code validity by code' do
  #         tags TAGS_API_DMO_ENTITY_CODES
  #         produces 'application/json'
  #         consumes 'application/json'
  #         parameter name: :params, in: :body, schema: {
  #           type: :object,
  #           properties: {
  #             code: { type: :string },
  #             required: [:code]
  #           }
  #         }
  #         security [Bearer: {}]

  #         response '200', 'entity code checked' do
  #           let(:Authorization) { "Bearer #{token_for(entreprise)}" }
  #           let!(:entity_code) { create(:entity_code, code: 'ABCD1234') }
  #           let(:params) do
  #             { code: 'ABCD1234' }
  #           end

  #           include_context 'with integration test'
  #           include_context 'not containings errors'
  #           after do |_example|
  #             json = JSON.parse(response.body)
  #             expect(json['success']).to eq true
  #             expect(json['code']).to eq 'ABCD1234'
  #             expect(json['message']).to eq 'Code valide et non expiré.'
  #           end
  #         end
  #       end
  #     end
  #   end
  # end
end
